/**
 * @file   int_stack.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Mon Mar 25 21:40:01 2002
 * 
 * @brief  stacks of integers
 * 
 * Note that we break the general two character prefix convention here
 * because is_ seems like a pretty confusing abbreviation
 */

#include <stdlib.h>
#include "int_stack.h"
#include "error_message.h"
#include "nalloc.h"
#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif
#include "memalloc.h"

na_node_p istack_nalloc(na_node_p node, slot_t slot, natural_t size){
  int_stack_p new;

  node=na_alloc(node,slot,sizeof(int_stack_t)+size*sizeof(int));
  new=na_access(node,slot);
  new->size=size;
  new->top=0;
  return node;
}


int_stack_p istack_new(natural_t size){
  int_stack_p new;

  new=memalloc(sizeof(int_stack_t)+size*sizeof(int));
  new->size=size;

  new->top=0;
  return new;
}

void istack_destroy(int_stack_p p){
  free(p);
}

int istack_empty(int_stack_p S){
  return (S->top == 0);
}

int istack_full(int_stack_p S){
  return ( S->top  >= S->size);
}

int istack_count(int_stack_p S){
  return S->top;
}

int istack_pop(int_stack_p S){
  int r;

  if (istack_empty(S)){
    err_fatal(HERE,"Stack empty");
  }
  r=S->elem[S->top-1];
  S->top--;
  return r;
}

void istack_flush(int_stack_p S){
  S->top=0;
}

int istack_top(int_stack_p S){
  int r;

  if (istack_empty(S)){
    err_fatal(HERE,"Stack empty");
  }
  r=S->elem[S->top];
  return r;
}

void istack_push(int_stack_p S, int i){
  if (istack_full(S)){
    err_fatal(HERE,"Stack full");
  }

  S->elem[S->top]=i;
  S->top++;
}


