/*
 * keep track of various events.
 */



#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifdef HAVE_CONFIG_H
#include "ssds_config.h"
#endif

#include "ssds_debug.h"
#ifdef USE_ZRAM
#include <z_par.h>
#endif

#include "rusage.h"
#include "memalloc.h"
#define PROFILE_LABELS
#include "profile.h"
#include "prof_struct.h"
#include "error_message.h"

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

static void prof_dump_profile(prof_profile_p prof_dat,char * label);

static prof_profile_p profile;

static int zram_running=0;

/**
   \todo, this needs to run the collect job to work under zram
*/
unsigned long prof_get(prof_event_t event){
  return profile->counter[event];
}

void prof_event(int event){
  if (event < 0 || event >=PROF_NUM_COUNTERS)
    err_fatal("prof_event","bogus profile event %d",event);

  profile->counter[event]++;
}

#ifdef USE_ZRAM

/* this is based on ../zram/nq_main.c. Look there for comments */

static z_JobNum prof_JobNum;
static int prof_profile_tag;

void CollectProfileJob(void)
{
  /* This procedure, which is called on every processor at the end of the
     computation, sends the local count of solutions back to the front end.
     */
  z_AddNodeBuffer((z_NodeP)profile);
} /* CollectProfileJob */


void prof_zram_init(void){
  prof_profile_tag=z_InstallClass(prof_node_t,0,0,0);
  prof_JobNum=z_InstallJob(CollectProfileJob);
  profile=(prof_profile_p)z_NewNode(prof_profile_tag,sizeof(prof_profile_t));
  zram_running=1;
}


/* This node is called on the front end with the returned data 
 from CollectProfileJob */
void AddNodeProfile(int pe){

  prof_profile_p node_prof;

  node_prof=(prof_profile_p) z_GetNodeBuffer();
  if (profile->level>1){
    char label[BUFSIZ];
    snprintf(label,BUFSIZ,"Node %d",pe);
    prof_dump_profile(node_prof,label);
  }

  if (pe!= z_myproc){
    int i;
    for (i=0; i< PROF_NUM_COUNTERS; i++){
      err_debug(SD_PROFILE,("\nAddNodeProfile","ctr %d before %d adding %d",
		   i,profile->counter[i],node_prof->counter[i]));

      profile->counter[i] += node_prof->counter[i];
    }
  }
  z_FreeNode((z_NodeP)node_prof);
}

#endif



void prof_init(){

  if (!zram_running)
    profile=memalloc(sizeof(prof_profile_t));

  prof_reset();
}

void prof_reset(){
  int i;
  profile->start=time(NULL);
  for (i=0; i< PROF_NUM_COUNTERS; i++)
    profile->counter[i]=0;
}
static void prof_dump_profile(prof_profile_p prof_dat,char * label){
  int i;



 fprintf(stderr,"\nExecution profile: %s\n",label);
 fprintf(stderr,"Wall Time: %ld seconds\n",(long)profile->stop-(long)profile->start);
  for (i=0; i< PROF_NUM_COUNTERS; i++){
    if (prof_dat->counter[i]>0 || profile->level>2)
      fprintf(stderr,"%s: %ld\n",profile_label[i],prof_dat->counter[i]);
  }	   
}

void prof_dump(int level){

  profile->stop=time(NULL);
  profile->level=level;

#ifdef USE_ZRAM  
  if (zram_running){
    z_StartParallelJob(prof_JobNum,AddNodeProfile);
    z_WaitWork();
  }
#endif


  if (profile->level > 1){
    prof_dump_profile(profile,"Total");
    ru_print();
  }

  /*   if (!zram_running)
       free(profile); */
}

