/**
 * @file   int_to_symbol.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Fri Dec 27 18:30:37 2002
 * 
 * @brief  convert an integer representing a sign to a symbol
 * 
 * this is the non-uniform version
 */
#include <stdlib.h>
#include "nu_sign.h"
#include "error_message.h"
#include "oms_types.h"
char *sign2string(nu_sign_t sigma){
  switch (sigma){
  case SIGN_POS:
    return "1";
    break;
  case SIGN_NEG:
    return "-1";
    break;
  case SIGN_ZERO:
    return "0";
    break;
  case SIGN_NONPOS:
    return "<=";
    break;
  case SIGN_NONNEG:
    return ">=";
    break;
  case SIGN_NONZERO:
    return "<>";
    break;
  case SIGN_NONE:
    return "%";
    break;
  case SIGN_ANY:
    return "?";
    break;
  default:
    err_fatal("sign_int","cannot convert sign '%s' to integer\n",
	      sign_name[sigma]);
  }
  /*NOTREACHED*/
}

char sign2char(nu_sign_t s){
    switch (s){
    case SIGN_NEG:
      return '-';
    case SIGN_ZERO:
      return '0';
    case SIGN_POS:
      return '+';
    case SIGN_ANY:
      return '?';
    case SIGN_NONE:
      return '%';
    case SIGN_NONZERO:
      return '^';
    case SIGN_NONNEG:
      return '>';
    case SIGN_NONPOS:
      return '<';
    default:
      err_fatal(HERE,"Unknown sign encoding %x",s);
      exit(1);
    }
}
char sign2prefix(nu_sign_t s){
    switch (s){
    case SIGN_NEG:
      return '-';
    case SIGN_ZERO:
      return '@';
    case SIGN_POS:
      return '+';
    case SIGN_ANY:
      return '?';
    case SIGN_NONE:
      return '%';
    case SIGN_NONZERO:
      return '^';
    case SIGN_NONNEG:
      return '>';
    case SIGN_NONPOS:
      return '<';
    default:
      err_fatal(HERE,"Unknown sign encoding %x",s);
      exit(1);
    }
}

char bdy2char(int b){
  char c;
    switch (b){
    case -1:
      c='-';
      break;
    case 1:
      c='+';
      break;
    case 0:
      c='?';
      break;
    default:
      err_fatal(HERE,"unknown boundary state %d",b);
    }
    return c;
}

bool sign_unknown(nu_sign_t s){
  return (s==SIGN_ANY);
}
