/**
 * @file   extract_boundary.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Sat Jun 15 16:23:32 2002
 * 
 * @brief  grab the boundary from an mpc input file
 * 
 * 
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "parser.h"
#include "gparams.h"
#include "error_message.h"
#include "output.h"
#include "facet_list.h"
#include "runtime_options.h"


int main(int argc, char** argv){

  params_p params;
  sa_array_p chi,bdy;
  int i,bases;


  
  params=malloc(sizeof(params_t));
  prof_init();
  ro_process_args(argc,argv,params,SO_nonuniform);
  if (!parse_input(params))
    exit(1);

  bases=binomial(params->n,params->r);
  chi=sa_new(bases,0);
  for (i=0; i < bases; i++){
    sa_init(chi,i,SIGN_ANY);
  }




  for (i=0; i <nss_count(parser.basis_signs); i++){
    int bindex;
    nu_sign_t sigma;
    nu_signed_index_t si=nss_next(parser.basis_signs);
    
    bindex=nsi_index(si);
    sigma=nsi_sign(si);
    sa_init(chi,bindex,sigma);
  }

  output_start_om(params);
  output_chi(params,chi);
  /* note, this is broken for xml output */
  if (params->boundary_output){
    printf("\nboundary={\n");
    fl_output(parser.facets);
    printf("}\n");
  }
  output_end_om(params);
  return 0;


}

