/* z_enumps.h (ZRAM: Enumeration: PostScript output)
   Ambros Marzetta
   Must be linked with -lm option.

 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 12:14:52 -0400 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1996/06/03  19:01:41  marzetta
 * Initial revision
 *
 */

#ifndef _Z_ENUMPS_H_
#define _Z_ENUMPS_H_
#include "z_base.h"
#include <stdio.h>


/* The global file where the nodes are saved ------------------------------ */

void z_enumps_Open(const char *name);
/* Create a file with the given name for use by z_Save, z_Load and
   z_enumps_Rewind. */

void z_Save(z_NodeP global, z_NodeP node);
/* Save a node to the file. The first parameter (global) is not used.
   It is here to make the procedure compatible to the z_enum_ResultProc
   type.
   Precondition: the file must be open. */

z_NodeP z_Load(void);
/* Read a node from the file. Precondition: the file must have been rewound. */

void z_enumps_Rewind(int32 *elem_count);
/* This procedure must be called after all z_Save calls and before all
   z_Load calls. It is used by z_PrintPS. */


/* Creating PostScript output from the saved nodes ------------------------ */

/* The following variables may be set by the application: */
extern float z_enumps_ratio;	/* height / width of frame */
extern boolean z_enumps_frame;	/* draw a rectangular frame */
extern float z_enumps_borderFraction; /* space between frames */

typedef void (*z_enumps_HeaderProc) (z_NodeP, FILE *, float, float);
typedef void (*z_enumps_PictureProc) (z_NodeP, z_NodeP, FILE *,
					    float, float);

void z_PrintPS(z_NodeP global, FILE *psfile,
	       z_enumps_HeaderProc AppDepHeader,
	       z_enumps_PictureProc appDepPicture);
	       /*void (*AppDepHeader) (z_NodeP, FILE *, float, float),*/
	       /*void (*AppDepPicture) (z_NodeP, z_NodeP, FILE *, float, float));*/
/* Rewind the file containing the saved nodes and create PostScript output
   from them.
   Pre: psfile is open for writing. */

/* AppDepHeader(global, psfile, xsize, ysize)
   {
     Write an application dependent PostScript header into psfile.
   }

   AppDepPicture(global, node, psfile, xsize, ysize)
   {
     Produce PostScript output for node into psfile. Coordinates are unscaled.
     The picture should be in the rectangle bounded by (0,0) and
     (xsize, ysize).
   }
*/


/* Initialization ---------------------------------------------------------- */

void z_enumps_init(void);

#endif /* _Z_ENUMPS_H_ */
