/* z_io.h (parallel io)

   Ambros Marzetta

 * $Source$
 * $Revision: 2773 $
 * $Date: 2001-09-02 09:59:32 -0300 (Sun, 02 Sep 2001) $
 * $Log$
 * Revision 1.5  2001/09/02 12:59:32  bremner
 * snapshot
 *
 * Revision 1.4  2001/08/29 20:00:28  bremner
 * reorganizing checkpointing
 *
 * Revision 1.3  2001/08/17 13:24:41  bremner
 * checkpointing not obviously broken
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.2  1996/05/15  16:54:49  marzetta
 * \Exported z_io_AppendProc
 *
 * Revision 1.1  1996/05/08  20:29:30  marzetta
 * Initial revision
 *
 */

#ifndef _Z_IO_H_
#define _Z_IO_H_

#include <stdio.h>
#include "z_base.h"

#ifndef _GETOPT_H_
#define _GETOPT_H_

extern int getopt(int, char * const [], const char *);
extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;

#endif /* _GETOPT_H_ */

#if 0
#ifndef PATH_MAX
#define PATH_MAX 255
#endif
#endif

/* Open / Close -------------------------------------------------------- */

void z_io_RedirectStdout(const char *path);
/* (FE only) Redirect stdout to a file (different for each processor) */

void z_io_RedirectStderr(const char *path);
/* (FE only) Redirect stderr to a file (different for each processor) */

int z_io_MakeBackup(const char *path);
/* Rename the file path.nnn to path.nnn.bak where nnn is the processor
   number. */

FILE *z_fopen(const char *path, const char *mode);
/* Open the file named by the path parameter with a processor number dependent
   extension. */

void z_io_AppendProc(char *s, const char *path, int numprocs, int myproc);
/* Append the processor number to the path (s := path.nnn). */


/* Read / Write -------------------------------------------------------- */

long z_fread(void *pointer, long size, long num_items, FILE *stream);
/* Same as fread() but aborts on error and eof */

void z_WriteNode(FILE *f, z_NodeP node);
/* Write the node to the file. node may be NULL. */

z_NodeP z_ReadNode(FILE *f);

/* Debugging */
void z_printf(const char *fmt, ...);
void z_fatal(const char *where, const char *fmt, ...);
void z_perror(const char *where, const char *fmt, ...);

#ifndef HERE
#define __here(l) __FILE__ ":" #l
#define _here(l) __here(l)
#define HERE _here(__LINE__)
#endif
/* Initialization ------------------------------------------------------ */

void z_io_Init(void);

#endif /* _Z_IO_H_ */
