/* z_md.h (definition of machine dependent part)

   Ambros Marzetta
   95-07-25

 * $Source$
 * $Revision: 2853 $
 * $Date: 2002-04-03 07:50:54 -0400 (Wed, 03 Apr 2002) $
 * $Log$
 * Revision 1.3  2002/04/03 11:50:53  bremner
 * tracking ugly memory leaks
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.6  1996/04/09  13:22:27  marzetta
 * Added Cenju-3, SEQUENTIAL
 *
 * Revision 1.5  1995/08/22  12:39:45  marzetta
 * New message passing, MPI
 *
 * Revision 1.3  1995/07/28  14:05:58  marzetta
 * Some cosmetic changes
 *
 * Revision 1.2  1995/07/28  08:34:52  marzetta
 * struct buffer_header, z_md_Init with path parameter for .lod-file
 *
 * Revision 1.1  1995/07/27  11:40:41  marzetta
 * Initial revision
 *
 */

#ifndef _Z_MD_H_
#define _Z_MD_H_
#include "z_base.h"
#include "z_par.h"		/* z_tagT */

extern  long z_bufunits;	/* defined / initialized in z_buf.c */
				/* size of buffer in unitT */

typedef MINT unitT;		/* must not be greater than MINT */
#define UNITS(x) (((x) + (sizeof(unitT)-1)) / sizeof(unitT))
				/* number of units for x bytes / dsp words */

/* Messages start with tags of type z_tagT, stored in buffer->header.msg_tag.

   TOKEN: is sent only when a processor is idle
   TERMTAG: is sent/received only when all processors are idle
   user data:

   */

struct buffer_header {
  z_tagT msg_tag;
  unsigned long len;		/* used only on Cenju-3 */
  int16 treeRoot;		/* used only with MPI broadcasts */
};

struct buffer {
  struct buffer_header header;
  unitT b[10];
};

#define for_all_nodes(i) for (i = z_numprocs - 1; i >= 0; i --)
				/* i must be signed */

/* Starting jobs ---------------------------------------------------- */
#ifndef SEQUENTIAL

void z_md_StartParallel(void);
/* (FE only) Start a parallel job on all processors.
   The tag of outbuffer contains the following information:
   - which job to start
   - whether all PEs return a result or only PE 0
   */

void z_md_RecBuffer(void);
/* (PE only) Wait until a message from z_StartParallelJob is received
   in inbuffer.
   Called by z_pe_loop, MusicStartParallel. */

/* Ending jobs ------------------------------------------------------- */

void z_md_Wait(z_ResultProc resultProc);
/* (FE only) Called by z_WaitWork.
   Wait for those other processors which should send a message in
   z_md_SendToFE. */

void z_md_SendToFE(boolean collect);
/* (PE only) Called by z_pe_switch if !z_frontend.
   Send the result of the computation to the front end which receives
   it in z_md_Wait.
   */

/* Message passing ------------------------------------------------ */

extern const boolean z_md_doesTD;
/* 0 -> Termination detection must be done by z_mp.c
        Return value of z_md_WaitRecv is ignored.
   1 -> Termination detection is done in z_md____.c
   */

void z_md_Isend(struct buffer *buffer, long size,
		 int dest, const char *comment);
/* May be implemented as Csend (bug in MPI implementation on paragon) */
/* Size is in sizeof() units and includes the header. */

void z_md_Ibroadcast(struct buffer *buffer, long size, const char *comment);
/* May be implemented as Csend (bug in MPI implementation on paragon) */

boolean z_md_TestSend(void);
/* Only called once after z_md_Isend and z_md_Ibroadcast */

void z_md_Irecv(void);

boolean z_md_TestRecv(void);

boolean z_md_WaitRecv(void);
/* Wait for the Irecv to complete or return true if all processors are idle. */

#endif /* SEQUENTIAL */
/* Assert ---------------------------------------------------------- */

void z_md_abort(void);
/* Called by assert() */

/* Initialization --------------------------------------------------- */

void z_md_Init(int *argc, char ***argv);
/* Called by main(). Calls z_par_Init(). */

void z_md_Fin(void);
/* Called my z_par_Fin */

#endif /* _Z_MD_H_ */
