/* z_ps2d.c

   ZRAM: 2-dimensional point set, used by the EST and CIS programs

   Ambros Marzetta

   95-07-15
 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 12:14:52 -0400 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1996/06/03  19:02:29  marzetta
 * Initial revision
 *
 */

#include <stdlib.h>		/* rand() */
#include "z_ps2d.h"

z_ps2dP z_ps2d_New(int numPt)
{
  z_ps2dP n;

  n = (z_ps2dP) z_NewNode(z_ps2dTag,
			  sizeof(z_ps2dT) + 2*numPt*sizeof(ps2d_coordT));
  if (!n) perror("z_ps2d_New");
  n->numPt = numPt;
  return n;
} /* z_ps2d_NewNode */

void z_ps2d_Free(z_ps2dP node)
{
  z_FreeNode((z_NodeP) node);
} /* z_ps2d_FreeNode */

void z_ps2d_ReadStart(z_ps2dP ps2d)
{
  int i;
  for (i=0;i<ps2d->numPt;i++) {
    ps2d_coord_x(ps2d, i) = rand() % 1000;
    ps2d_coord_y(ps2d, i) = rand() % 1000;
  }
} /* z_ps2d_ReadStart */

void z_ps2d_Read(FILE *infile,
		 z_ps2dP ps2d)
{
  int p;
  ps2d_coordT x, y;
  if(fscanf(infile, "%d:%f%f", &p, &x, &y) != 3) {
    fprintf(stderr, "z_ps2d_Read: Error in coordinate\n");
    return;
  }
  if (p >= 0 && p < ps2d->numPt) {
    ps2d_coord_x(ps2d, p) = x;
    ps2d_coord_y(ps2d, p) = y;
  } else
    fprintf(stderr, "z_ps2d_Read: Bad point for coordinate\n");
} /* z_ps2d_Read */

static boolean z_ps2d_inited;
int z_ps2dTag;

void z_ps2d_Init(void)
{
  if (! z_ps2d_inited) {
    z_ps2d_inited = 1;
    z_ps2dTag = z_InstallClass(z_ps2d, 0, 0, 0);
  }
} /* z_ps2d_Init */
