/* z_signal.c (ZRAM: signal handler)
   Ambros Marzetta

 * $Source$
 * $Revision: 2760 $
 * $Date: 2001-08-26 23:44:49 -0300 (Sun, 26 Aug 2001) $
 * $Log$
 * Revision 1.5  2001/08/27 02:44:49  bremner
 * snapshot
 *
 * Revision 1.4  2001/08/18 03:07:35  bremner
 * checkpointing seems to work
 *
 * Revision 1.3  2001/08/10 20:41:42  bremner
 * starting to get checkpointing working
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.2  1996/05/15  16:43:58  marzetta
 * Moved an #ifdef macintosh
 *
 * Revision 1.1  1996/05/08  20:30:42  marzetta
 * Initial revision
 *
 */

#include "z_base.h"
#include "z_par.h"
#include "z_signal.h"
#include "z_time.h"
#include "stdio.h"
#include <errno.h>
#include <signal.h>
#ifndef macintosh
#include <sys/time.h>
#endif /* macintosh */

boolean z_signal_trace=1;	/* trace the signal handler */
				/* warning: may interrupt any output */
/* long z_signal_checkpointTime=ZRAM_CHECKPOINT_SECONDS; */
                                /* time in seconds between checkpoints */

/* Signal handler ----------------------------------------------------- */

#ifndef macintosh
static void z_term_handler(int signal)
{
  if (z_signal_trace)
    fprintf(stderr,"\n[%2d]Got signal %d (time=%f z_checkpoint=%d)\n",
	   z_myproc, signal, z_Wtime(),z_checkpoint);
  /*  fflush(stdout); */
  /* Is it allowed to call fflush from a signal handler?
     Is fflush reentrant? */
  z_terminate = (signal == SIGTERM);
  z_checkpoint = 1;
} /* z_term_handler */
#endif

void z_StartCheckpointTimer(long timeout)
{
#if !defined(macintosh) && !defined(cenju3)
  struct itimerval new;

  new.it_interval.tv_sec = timeout;
  new.it_interval.tv_usec = 0;
  new.it_value.tv_sec = timeout;
  new.it_value.tv_usec = 0;
  
  if (setitimer(ITIMER_REAL, &new, NULL))
    perror("setitimer");
#endif
} /* z_StartCheckpointTimer */


/* Initialization ----------------------------------------------------- */

void z_signal_Init(void)
{
  /* Install a signal handler for the terminate signal. When the process
     receives the signal, it sets the z_terminate and z_checkpoint variables.
     */

#if !defined(macintosh) && !defined(cenju3)
  static struct sigaction action, o_action;
  action.sa_handler = z_term_handler;
  /* sigset_t is defined differently depending on the operation system.
     declaring the struct as static simplifies the initialization. */
  
  if (sigaction(SIGALRM, &action, &o_action))
    perror("sigaction");
  if (sigaction(SIGTERM, &action, &o_action))
    perror("sigaction");
#endif
} /* z_signal_Init */


void z_signal_Block(void){
#if !defined(macintosh) && !defined(cenju3)
  
  sigset_t sigset,oldsigset;
  sigemptyset(&sigset);
  if(sigaddset(&sigset,SIGALRM)) {
    perror("sigaddset");
    abort();
  };

  if (sigprocmask(SIG_BLOCK,&sigset,&oldsigset)){
    perror("sigprocmask");
    abort();
  }

#endif
}


void z_signal_UnBlock(void){
#if !defined(macintosh) && !defined(cenju3)
  
  sigset_t sigset;
  
  sigemptyset(&sigset);
  sigaddset(&sigset,SIGALRM);
  if (sigprocmask(SIG_UNBLOCK,&sigset,NULL)){
    perror("sigprocmask");
    abort();
  }

#endif
}


/* Comments ----------------------------------------------------------- */

/* In OSF/1 the functions signal(), sigvec() and sigblock() are obsolete.
   */
